@@ -12,7 +12,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -23,6 +23,28 @@
  * Software Foundation; either version 2, or (at your option) any later 
  * version.
  * 
+ * Linking GNU Smalltalk statically or dynamically with other modules is
+ * making a combined work based on GNU Smalltalk.  Thus, the terms and
+ * conditions of the GNU General Public License cover the whole
+ * combination.
+ *
+ * In addition, as a special exception, the Free Software Foundation
+ * give you permission to combine GNU Smalltalk with free software
+ * programs or libraries that are released under the GNU LGPL and with
+ * independent programs running under the GNU Smalltalk virtual machine.
+ *
+ * You may copy and distribute such a system following the terms of the
+ * GNU GPL for GNU Smalltalk and the licenses of the other code
+ * concerned, provided that you include the source code of that other
+ * code when and as the GNU GPL requires distribution of source code.
+ *
+ * Note that people who make modified versions of GNU Smalltalk are not
+ * obligated to grant this special exception for their modified
+ * versions; it is their choice whether to do so.  The GNU General
+ * Public License gives permission to release a modified version without
+ * this exception; this exception also makes it possible to release a
+ * modified version which carries forward this exception.
+ *
  * GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
@@ -100,7 +122,15 @@ VMProxy gst_interpreter_proxy = {
   _gst_class_is_kind_of, _gst_object_is_kind_of,
   _gst_perform, _gst_perform_with, _gst_class_implements_selector,
   _gst_class_can_understand, _gst_responds_to,
-  _gst_oop_size, _gst_oop_at, _gst_oop_at_put
+  _gst_oop_size, _gst_oop_at, _gst_oop_at_put,
+
+  /* System objects.  */
+  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+
+  /* New in 2.3.  */
+  _gst_wchar_to_oop, _gst_wstring_to_oop,
+  _gst_oop_to_wchar, _gst_oop_to_wstring,
 };
 
 OOP
@@ -313,6 +343,16 @@ _gst_msg_sendf (PTR resultPtr,
 	    INC_ADD_OOP (args[i]);
           }
           break;
+
+        case 'w':
+          args[++i] = char_new ((char) va_arg (ap, wchar_t));
+	  INC_ADD_OOP (args[i]);
+          break;
+
+        case 'W':
+          args[++i] = _gst_unicode_string_new (va_arg (ap, const wchar_t *));
+	  INC_ADD_OOP (args[i]);
+          break;
 	}
     }
 
@@ -363,6 +403,16 @@ _gst_msg_sendf (PTR resultPtr,
 	  *(long *) resultPtr = _gst_oop_to_c (result);
 	  break;
 
+	case 'w':
+	  *(wchar_t *) resultPtr =
+	    IS_NIL (result) ? 0 : CHAR_OOP_VALUE (result);
+	  break;
+
+	case 'W':
+	  *(wchar_t **) resultPtr =
+	    IS_NIL (result) ? NULL : _gst_to_wide_cstring (result);
+	  break;
+
 	case 'o':
 	default:
 	  *(OOP *) resultPtr = result;
@@ -446,9 +496,25 @@ OOP
 _gst_class_name_to_oop (const char *name)
 {
   OOP result, key;
+  char *s, *p, *prev_p;
 
-  key = _gst_symbol_to_oop (name);	/* this inits Smalltalk */
-  result = dictionary_at (_gst_smalltalk_dictionary, key);
+  if (!name || !*name)
+    return NULL;
+
+  s = strdup (name);
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  result = _gst_smalltalk_dictionary;
+  for (p = s; (prev_p = strsep (&p, ".")) != NULL; )
+    {
+      key = _gst_intern_string (prev_p);
+      result = dictionary_at (result, key);
+      if (IS_NIL (result))
+	return NULL;
+    }
+
+  free (s);
   return (result);
 }
 
@@ -505,6 +571,15 @@ _gst_char_to_oop (char c)
   return (CHAR_OOP_AT (c));
 }
 
+OOP
+_gst_wchar_to_oop (wchar_t wc)
+{
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  return (char_new (wc));
+}
+
 
 /* !!! Add in byteArray support sometime soon */
 
@@ -520,6 +595,18 @@ _gst_string_to_oop (const char *str)
     return (INC_ADD_OOP (_gst_string_new (str)));
 }
 
+OOP
+_gst_wstring_to_oop (const wchar_t *str)
+{
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  if (str == NULL)
+    return (_gst_nil_oop);
+  else
+    return (INC_ADD_OOP (_gst_unicode_string_new (str)));
+}
+
 OOP
 _gst_byte_array_to_oop (const char *str,
 			int n)
@@ -590,7 +677,8 @@ _gst_oop_to_c (OOP oop)
 	   || OOP_CLASS (oop) == _gst_false_class)
     return (oop == _gst_true_oop);
 
-  else if (OOP_CLASS (oop) == _gst_char_class)
+  else if (OOP_CLASS (oop) == _gst_char_class
+           || OOP_CLASS (oop) == _gst_unicode_character_class)
     return (CHAR_OOP_VALUE (oop));
 
   else if (IS_NIL (oop))
@@ -671,6 +759,15 @@ _gst_oop_to_char (OOP oop)
   return (CHAR_OOP_VALUE (oop));
 }
 
+wchar_t
+_gst_oop_to_wchar (OOP oop)
+{
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  return (CHAR_OOP_VALUE (oop));
+}
+
 char *
 _gst_oop_to_string (OOP oop)
 {
@@ -683,6 +780,18 @@ _gst_oop_to_string (OOP oop)
     return ((char *) _gst_to_cstring (oop));
 }
 
+wchar_t *
+_gst_oop_to_wstring (OOP oop)
+{
+  if (!_gst_smalltalk_initialized)
+    gst_init_smalltalk ();
+
+  if (IS_NIL (oop))
+    return (NULL);
+  else
+    return ((wchar_t *) _gst_to_wide_cstring (oop));
+}
+
 char *
 _gst_oop_to_byte_array (OOP oop)
 {
@@ -865,7 +974,7 @@ _gst_oop_at_put (OOP oop, size_t index, OOP new)
 
   old = index_oop (oop, index + 1);
   assert (old);
-  index_oop_put (oop, index, new);
+  index_oop_put (oop, index + 1, new);
   return old;
 }
 
