@@ -7,7 +7,7 @@
 
 /***********************************************************************
  *
- * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002
+ * Copyright 1988,89,90,91,92,94,95,99,2000,2001,2002,2006
  * Free Software Foundation, Inc.
  * Written by Steve Byrne.
  *
@@ -18,6 +18,28 @@
  * Software Foundation; either version 2, or (at your option) any later 
  * version.
  * 
+ * Linking GNU Smalltalk statically or dynamically with other modules is
+ * making a combined work based on GNU Smalltalk.  Thus, the terms and
+ * conditions of the GNU General Public License cover the whole
+ * combination.
+ *
+ * In addition, as a special exception, the Free Software Foundation
+ * give you permission to combine GNU Smalltalk with free software
+ * programs or libraries that are released under the GNU LGPL and with
+ * independent programs running under the GNU Smalltalk virtual machine.
+ *
+ * You may copy and distribute such a system following the terms of the
+ * GNU GPL for GNU Smalltalk and the licenses of the other code
+ * concerned, provided that you include the source code of that other
+ * code when and as the GNU GPL requires distribution of source code.
+ *
+ * Note that people who make modified versions of GNU Smalltalk are not
+ * obligated to grant this special exception for their modified
+ * versions; it is their choice whether to do so.  The GNU General
+ * Public License gives permission to release a modified version without
+ * this exception; this exception also makes it possible to release a
+ * modified version which carries forward this exception.
+ *
  * GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
@@ -140,10 +162,14 @@ extern OOP _gst_bool_to_oop (int b)
   ATTRIBUTE_HIDDEN;
 extern OOP _gst_char_to_oop (char c) 
   ATTRIBUTE_HIDDEN;
+extern OOP _gst_wchar_to_oop (wchar_t c) 
+  ATTRIBUTE_HIDDEN;
 extern OOP _gst_class_name_to_oop (const char *name) 
   ATTRIBUTE_HIDDEN;
 extern OOP _gst_string_to_oop (const char *str) 
   ATTRIBUTE_HIDDEN;
+extern OOP _gst_wstring_to_oop (const wchar_t *str) 
+  ATTRIBUTE_HIDDEN;
 extern OOP _gst_byte_array_to_oop (const char *str,
 				   int n) 
   ATTRIBUTE_HIDDEN;
@@ -171,8 +197,12 @@ extern int _gst_oop_to_bool (OOP oop)
   ATTRIBUTE_HIDDEN;
 extern char _gst_oop_to_char (OOP oop) 
   ATTRIBUTE_HIDDEN;
+extern wchar_t _gst_oop_to_wchar (OOP oop) 
+  ATTRIBUTE_HIDDEN;
 extern char *_gst_oop_to_string (OOP oop) 
   ATTRIBUTE_HIDDEN;
+extern wchar_t *_gst_oop_to_wstring (OOP oop) 
+  ATTRIBUTE_HIDDEN;
 extern char *_gst_oop_to_byte_array (OOP oop) 
   ATTRIBUTE_HIDDEN;
 extern PTR _gst_oop_to_cobject (OOP oop) 
