@@ -18,6 +18,28 @@
  * Software Foundation; either version 2, or (at your option) any later 
  * version.
  * 
+ * Linking GNU Smalltalk statically or dynamically with other modules is
+ * making a combined work based on GNU Smalltalk.  Thus, the terms and
+ * conditions of the GNU General Public License cover the whole
+ * combination.
+ *
+ * In addition, as a special exception, the Free Software Foundation
+ * give you permission to combine GNU Smalltalk with free software
+ * programs or libraries that are released under the GNU LGPL and with
+ * independent programs running under the GNU Smalltalk virtual machine.
+ *
+ * You may copy and distribute such a system following the terms of the
+ * GNU GPL for GNU Smalltalk and the licenses of the other code
+ * concerned, provided that you include the source code of that other
+ * code when and as the GNU GPL requires distribution of source code.
+ *
+ * Note that people who make modified versions of GNU Smalltalk are not
+ * obligated to grant this special exception for their modified
+ * versions; it is their choice whether to do so.  The GNU General
+ * Public License gives permission to release a modified version without
+ * this exception; this exception also makes it possible to release a
+ * modified version which carries forward this exception.
+ *
  * GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
@@ -2132,7 +2154,7 @@ make_constant_oop (tree_node constExpr)
   OOP resultOOP, elementOOP;
   inc_ptr incPtr;
   byte_object bo;
-  mst_Object result;
+  gst_object result;
 
   if (constExpr == NULL)
     return (_gst_nil_oop);	/* special case empty array literals */
@@ -2311,7 +2333,7 @@ _gst_make_attribute (tree_node attribute_keywords)
 {
   tree_node keyword;
   OOP selectorOOP, argsArrayOOP, messageOOP;
-  mst_Object argsArray;
+  gst_object argsArray;
   int i, numArgs;
   inc_ptr incPtr;
 
@@ -2426,7 +2448,7 @@ process_attribute (OOP messageOOP)
   gst_message message = (gst_message) OOP_TO_OBJ (messageOOP);
   OOP selectorOOP = message->selector;
   OOP argumentsOOP = message->args;
-  mst_Object arguments = OOP_TO_OBJ (argumentsOOP);
+  gst_object arguments = OOP_TO_OBJ (argumentsOOP);
 
   if (selectorOOP == _gst_primitive_symbol)
     {
@@ -2529,7 +2551,7 @@ OOP
 get_literals_array (void)
 {
   OOP methodLiteralsOOP;
-  mst_Object methodLiterals;
+  gst_object methodLiterals;
 
   assert (literal_vec_curr > literal_vec);
 
@@ -2725,7 +2747,7 @@ method_new (method_header header,
   int numByteCodes;
   gst_compiled_method method;
   OOP methodOOP, methodDesc;
-  mst_Object lit;
+  gst_object lit;
   int i;
 
   if (bytecodes != NULL)
@@ -2855,45 +2877,29 @@ method_info_new (OOP class,
 OOP
 file_segment_new (void)
 {
-  OOP fileName, stringContents, fileSegmentOOP;
+  OOP fileName, fileSegmentOOP;
   gst_file_segment fileSegment;
-  int64_t startPos;
+  int64_t startPos = _gst_get_method_start_pos ();
   inc_ptr incPtr;
 
-  switch (_gst_get_cur_stream_type ())
-    {
-    case STREAM_UNKNOWN:
-    default:
-      return (_gst_nil_oop);
+  if (startPos == -1)
+    return _gst_get_cur_string ();
 
-    case STREAM_FILE:
-      incPtr = INC_SAVE_POINTER ();
-      fileName = _gst_get_cur_file_name ();
-      INC_ADD_OOP (fileName);
+  incPtr = INC_SAVE_POINTER ();
+  fileName = _gst_get_cur_file_name ();
+  INC_ADD_OOP (fileName);
 
-      fileSegment = (gst_file_segment) new_instance (_gst_file_segment_class,
-    						 &fileSegmentOOP);
+  fileSegment = (gst_file_segment) new_instance (_gst_file_segment_class,
+						 &fileSegmentOOP);
 
-      fileSegment->fileName = fileName;
-      startPos = _gst_get_method_start_pos ();
-      fileSegment->startPos = from_c_int_64 (startPos);
-      fileSegment->length =
-        from_c_int_64 (_gst_get_cur_file_pos () - startPos - 1);
+  fileSegment->fileName = fileName;
+  fileSegment->startPos = from_c_int_64 (startPos);
+  fileSegment->length =
+    from_c_int_64 (_gst_get_cur_file_pos () - startPos - 1);
 
-      assert (to_c_int_64 (fileSegment->length) >= 0);
-      INC_RESTORE_POINTER (incPtr);
-      return (fileSegmentOOP);
-
-    case STREAM_STRING:
-      stringContents = _gst_get_cur_string ();
-      return (stringContents);
-
-#ifdef HAVE_READLINE
-    case STREAM_READLINE:
-      stringContents = _gst_get_cur_readline ();
-      return (stringContents);
-#endif /* HAVE_READLINE */
-    }
+  assert (to_c_int_64 (fileSegment->length) >= 0);
+  INC_RESTORE_POINTER (incPtr);
+  return (fileSegmentOOP);
 }
 
 void _gst_restore_primitive_number (OOP methodOOP, int *map)
