@@ -18,6 +18,28 @@
  * Software Foundation; either version 2, or (at your option) any later 
  * version.
  * 
+ * Linking GNU Smalltalk statically or dynamically with other modules is
+ * making a combined work based on GNU Smalltalk.  Thus, the terms and
+ * conditions of the GNU General Public License cover the whole
+ * combination.
+ *
+ * In addition, as a special exception, the Free Software Foundation
+ * give you permission to combine GNU Smalltalk with free software
+ * programs or libraries that are released under the GNU LGPL and with
+ * independent programs running under the GNU Smalltalk virtual machine.
+ *
+ * You may copy and distribute such a system following the terms of the
+ * GNU GPL for GNU Smalltalk and the licenses of the other code
+ * concerned, provided that you include the source code of that other
+ * code when and as the GNU GPL requires distribution of source code.
+ *
+ * Note that people who make modified versions of GNU Smalltalk are not
+ * obligated to grant this special exception for their modified
+ * versions; it is their choice whether to do so.  The GNU General
+ * Public License gives permission to release a modified version without
+ * this exception; this exception also makes it possible to release a
+ * modified version which carries forward this exception.
+ *
  * GNU Smalltalk is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
@@ -56,7 +78,8 @@
 static const char help_text[] =
   "GNU Smalltalk usage:"
   "\n"
-  "\n    gst [ flag ... ] [ file ...]"
+  "\n    gst [ flag ... ] [ file ... ]"
+  "\n    gst [ flag ... ] { -f | --file } file [ args ... ]"
   "\n"
   "\nShort flags can appear either as -xyz or as -x -y -z.  If an option is"
   "\nmandatory for a long option, it is also mandatory for a short one. The"
@@ -86,11 +109,14 @@ static const char help_text[] =
   "\nFiles are loaded one after the other.  After the last one is loaded,"
   "\nSmalltalk will exit.  If no files are specified, Smalltalk reads from"
   "\nthe terminal, with prompts."
+  "\n"
+  "\nIn the second form, the file after -f is the last loaded file; any"
+  "\nparameter after that file is passed to the Smalltalk program."
   "\n" "\nReport bugs to help-smalltalk@gnu.org\n";
 
 static const char copyright_and_legal_stuff_text[] =
   "GNU Smalltalk version %s"
-  "\nCopyright 2003 Free Software Foundation, Inc."
+  "\nCopyright 2006 Free Software Foundation, Inc."
   "\nWritten by Steve Byrne (sbb@gnu.org) and Paolo Bonzini (bonzini@gnu.org)"
   "\n"
   "\nGNU Smalltalk comes with NO WARRANTY, to the extent permitted by law."
